/** 
 * Copyright (c) Krapht, 2011
 * 
 * "LogisticsPipes" is distributed under the terms of the Minecraft Mod Public 
 * License 1.0, or MMPL. Please check the contents of the license located in
 * http://www.mod-buildcraft.com/MMPL-1.0.txt
 */

package net.minecraft.src.buildcraft.krapht;

import java.util.HashMap;
import java.util.Set;

import net.minecraft.src.krapht.ItemIdentifier;

public class LogisticsManager {
	private class LogisticsValue {
		int TotalThisCycle = 0;
		int CompletedThisCycle = 0;
		float cycleCompletedFraction;

		public LogisticsValue(int TotalToDeliver) {
			this.TotalThisCycle = TotalToDeliver;
			recalc();
		}
		
		public void deliverItem() {
			CompletedThisCycle++;
			recalc();
		}
		
		public void setCompleted() {
			CompletedThisCycle = TotalThisCycle;
			recalc();
		}
		
		private void recalc(){
			cycleCompletedFraction = TotalThisCycle == 0 ? 1F : (1F / (float) TotalThisCycle) * (float) CompletedThisCycle;
		}
	}
	
	private static HashMap<ItemIdentifier, HashMap<Router, LogisticsValue >>  _logisticsDatabase = new HashMap<ItemIdentifier, HashMap<Router,LogisticsValue>>();
	private static LogisticsManager _instance;
	
	static { //Workaround to create instances of internal class
		_instance = new LogisticsManager();
	}
	
	public static Router getDestinationFor(ItemIdentifier item, Set<Router> validDestinations) {
		if (!_logisticsDatabase.containsKey(item)){
			_logisticsDatabase.put(item, new HashMap<Router, LogisticsValue>());
		}
		
		HashMap<Router, LogisticsValue> itemEntry = _logisticsDatabase.get(item);
		
		Router nextDestination = null;
		LogisticsValue nextDestinationValue = null;
		
		
		boolean allCompleted = true;
		for (Router r : validDestinations) {
			if (!itemEntry.containsKey(r)){
				itemEntry.put(r, _instance.new LogisticsValue(((PipeLogicLogistics)r.getPipe().logic).RequestsItem(item)));
			}
			LogisticsValue value = itemEntry.get(r);
			if (value.cycleCompletedFraction < 1F) {
				allCompleted = false;
				continue;
			}
		}
		
		if (allCompleted) {
			for (Router r : validDestinations){
				itemEntry.put(r, _instance.new LogisticsValue(((PipeLogicLogistics)r.getPipe().logic).RequestsItem(item)));
			}
		}
		
		for (Router r : validDestinations) {
			LogisticsValue value = itemEntry.get(r);
			
			//Ensure router still desires items
			int requested = ((PipeLogicLogistics)r.getPipe().logic).RequestsItem(item);
			if (requested == 0){
				value.setCompleted();
			}
			
			if (value.cycleCompletedFraction == 1F)	{
				continue;
			}
			
			if (nextDestinationValue == null || value.cycleCompletedFraction < nextDestinationValue.cycleCompletedFraction)	{
				nextDestinationValue = value;
				nextDestination = r;
			}
		}
		
		if (nextDestinationValue != null){
			nextDestinationValue.deliverItem();
		}
		return nextDestination;
	}
}
