/** 
 * Copyright (c) Krapht, 2011
 * 
 * "LogisticsPipes" is distributed under the terms of the Minecraft Mod Public 
 * License 1.0, or MMPL. Please check the contents of the license located in
 * http://www.mod-buildcraft.com/MMPL-1.0.txt
 */


/*
TODO: now
 - REMOVE DEBUG CODE (always)
 - Write manual / wiki
 
TODO later, maybe....
 - Add ingame item for network management (turn on/off link detection, poke link detection etc) ?
 - Context sensitive textures. Flashing routers on deliveries?
 - Track deliveries / en route ?
 - Save stuff, like destinations
 - Texture improvement
 - Persistance:
 	- Save logistics to file. Save coordinates so they can be resolved later. Also save items in transit and count them as not delivered
 - SMP:
	- Peering, transport other peoples items. Need hook to set owner of PassiveEntity
 */

package net.minecraft.src;

import java.io.File;

import net.minecraft.src.buildcraft.core.CoreProxy;
import net.minecraft.src.buildcraft.krapht.PipeItemsLogistics;
import net.minecraft.src.buildcraft.transport.BlockGenericPipe;
import net.minecraft.src.buildcraft.transport.Pipe;
import net.minecraft.src.forge.Configuration;
import net.minecraft.src.forge.MinecraftForgeClient;
import net.minecraft.src.forge.Property;

public class mod_LogisticsPipes extends BaseMod{
	
	public static Item LogisticsPipe;
	
	
	public static int LOGISTICSPIPE_ID						= 6874;
	public static int LOGISTICSPIPE_TEXTURE					= 0;
	public static final String LOGISTICSPIPE_TEXTURE_FILE	= "/net/minecraft/src/buildcraft/krapht/gui/logisticspipe.png";
	
	public static int LOGISTICS_DETECTION_LENGTH	= 50;
	public static int LOGISTICS_DETECTION_COUNT		= 100;
	public static int LOGISTICS_DETECTION_FREQUENCY = 20;
	
	private static Configuration configuration;
	
	@Override
	public void ModsLoaded() {
		super.ModsLoaded();
		
		File configFile = new File(CoreProxy.getBuildCraftBase(), "config/LogisticsPipes.cfg");
		configuration = new Configuration(configFile);
		configuration.load();
		
		Property logisticPipeIdProperty = configuration.getOrCreateIntProperty("logisticsPipe.id", Configuration.ITEM_PROPERTY, LOGISTICSPIPE_ID);
		logisticPipeIdProperty.comment = "The item id for the logistics pipe";
		
		Property detectionLength = configuration.getOrCreateIntProperty("detectionLength", Configuration.GENERAL_PROPERTY, LOGISTICS_DETECTION_LENGTH);
		detectionLength.comment = "The maximum shortest length between logistics pipes. This is an indicator on the maxim depth of the recursion algorithm to discover logistics neighbours. A low value might use less CPU, a high value will allow longer pipe sections";
		
		Property detectionCount = configuration.getOrCreateIntProperty("detectionCount", Configuration.GENERAL_PROPERTY, LOGISTICS_DETECTION_COUNT);
		detectionCount.comment = "The maximum number of buildcraft pipees (including forks) between logistics pipes. This is an indicator of the maximum ammount of nodes the recursion algorithm will visit before giving up. As it is possible to fork a pipe connection using standard BC pipes the algorithm will attempt to discover all available destinations through that pipe. Do note that the logistics system will not interfere with the operation of non-logistics pipes. So a forked pipe will usually be sup-optimal, but it is possible. A low value might reduce CPU usage, a high value will be able to handle more complex pipe setups. If you never fork your connection between the logistics pipes this has the same meaning as detectionLength and the lower of the two will be used";
		
		Property detectionFrequency = configuration.getOrCreateIntProperty("detectionFrequency", Configuration.GENERAL_PROPERTY, LOGISTICS_DETECTION_FREQUENCY);
		detectionFrequency.comment = "The amount of time that passes between checks to see if it is still connected to its neighbours. A low value will mean that it will detect changes faster but use more CPU. A high value means detection takes longer, but CPU consumption is reduced. A value of 20 will check about every second";
				
		configuration.save();
		 
		LOGISTICSPIPE_ID 				= Integer.parseInt(logisticPipeIdProperty.value);
		LOGISTICS_DETECTION_LENGTH		= Integer.parseInt(detectionLength.value);
		LOGISTICS_DETECTION_COUNT		= Integer.parseInt(detectionCount.value);
		LOGISTICS_DETECTION_FREQUENCY 	= Math.max(Integer.parseInt(detectionFrequency.value), 1);

		LOGISTICSPIPE_TEXTURE = CoreProxy.addCustomTexture(LOGISTICSPIPE_TEXTURE_FILE);
		
		MinecraftForgeClient.preloadTexture(LOGISTICSPIPE_TEXTURE_FILE);
		
		
		LogisticsPipe = createPipe(LOGISTICSPIPE_ID, PipeItemsLogistics.class, "Logistics Pipe");
		
		CraftingManager craftingmanager = CraftingManager.getInstance();
		craftingmanager.addRecipe(new ItemStack(LogisticsPipe, 8), new Object[] { "grg", "GdG", "grg", Character.valueOf('g'), Block.glass, 
																									   Character.valueOf('G'), BuildCraftCore.goldGearItem,
																									   Character.valueOf('d'), BuildCraftTransport.pipeItemsDiamond, 
																									   Character.valueOf('r'), Block.torchRedstoneActive});
		
		MinecraftForgeClient.registerCustomItemRenderer(LogisticsPipe.shiftedIndex, mod_BuildCraftTransport.instance);
		
	}
	
	@Override
	public String Version() {
		return "0.0.2 (Minecraft 1.8.1, Buildcraft 2.2.5, Forge 1.1.2)";
	}
	
	private static Item createPipe (int defaultID, Class <? extends Pipe> clas, String descr) {
		String name = Character.toLowerCase(clas.getSimpleName().charAt(0))
				+ clas.getSimpleName().substring(1);
		
		Item res =  BlockGenericPipe.registerPipe (defaultID, clas);
		res.setItemName(clas.getSimpleName());
		CoreProxy.addName(res, descr);
		MinecraftForgeClient.registerCustomItemRenderer(res.shiftedIndex, mod_BuildCraftTransport.instance);
	
		return res;
	}
}
